<?php
/**
 * Plugin Name: CIDRAM
 * Version: 2.2.0
 * Description: CIDRAM (Classless Inter-Domain Routing Access Manager) is a PHP script designed to protect websites by blocking requests originating from IP addresses regarded as being sources of undesirable traffic, including (but not limited to) traffic from non-human access endpoints, cloud services, spambots, scrapers, etc. It does this by calculating the possible CIDRs of the IP addresses supplied from inbound requests and then attempting to match these possible CIDRs against its signature files (these signature files contain lists of CIDRs of IP addresses regarded as being sources of undesirable traffic); If matches are found, the requests are blocked.
 * Author: Caleb Mazalevskis
 * Author URI: https://github.com/Maikuolan
 * Plugin URI: https://cidram.github.io/
 * Contributors: https://github.com/CIDRAM/CIDRAM/graphs/contributors
 * Tags: anti-spam, asn, blacklist, blocker, blocklist, cidr, cloud, firewall, subnet, subnets, gpl, ip, protection, routing, security, waf
 * License: GPLv2
 * License URI: https://github.com/CIDRAM/CIDRAM/blob/master/LICENSE.txt
 */

/** Only execute when not from within a cronjob! */
if (!wp_doing_cron()) {
    /** Load CIDRAM's main loader file. */
    require __DIR__ . '/loader.php';
}

/**
 * This function based upon the code supplied by jsdalton, here:
 * https://wordpress.org/support/topic/how-to-change-plugins-load-order#post-1385859
 * We use this to help ensure that CIDRAM is loaded before any other plugins, which is necessary to avoid conflicts
 * with a number of Wordpress caching plugins.
 */
function wpCIDRAM_LoadMeFirst() {
    $ThisPlugin = plugin_basename(trim(preg_replace('/(.*)plugins\/(.*)$/', WP_PLUGIN_DIR . '/$2', __FILE__)));
    $ActivePlugins = get_option('active_plugins');
    $Key = array_search($ThisPlugin, $ActivePlugins);
    if ($Key) {
        array_splice($ActivePlugins, $Key, 1);
        array_unshift($ActivePlugins, $ThisPlugin);
        update_option('active_plugins', $ActivePlugins);
    }
}
add_action('activated_plugin', 'wpCIDRAM_LoadMeFirst', 20);

/** Allows editing CIDRAM's INI and DAT files (necessary for editing configuration, signature files, etc. */
function wpCIDRAM_EditTypes($editable_extensions) {
    if (substr($_SERVER['QUERY_STRING'], 0, 11) === 'file=cidram') {
        array_unshift($editable_extensions, 'ini', 'dat', 'yaml');
    }
    return $editable_extensions;
}
add_filter('editable_extensions', 'wpCIDRAM_EditTypes');
